package org.foo.bar.mysql;

import java.sql.Driver;
import java.sql.SQLException;
import java.util.Properties;

import javax.sql.ConnectionPoolDataSource;
import javax.sql.DataSource;
import javax.sql.XADataSource;

import com.mysql.jdbc.jdbc2.optional.MysqlConnectionPoolDataSource;
import com.mysql.jdbc.jdbc2.optional.MysqlXADataSource;

/**
 * A factory for creating MySQL data sources. The properties specified in the
 * create methods determine how the created object is configured.
 * 
 * Sample code for obtaining a Derby network data source:
 * 
 * ServiceTracker tracker = new ServiceTracker(context,
 * DataSourceFactory.class.getName(), null); tracker.open(); DataSourceFactory
 * dsf = (DataSourceFactory) tracker.getService(); Properties props = new
 * Properties(); props.put(DataSourceFactory.JDBC_SERVER_NAME, "localhost");
 * props.put(DataSourceFactory.JDBC_PORT_NUMBER, "1527");
 * props.put(DataSourceFactory.JDBC_DATABASE_NAME, "myDB");
 * props.put(DataSourceFactory.JDBC_USER, "mike");
 * props.put(DataSourceFactory.JDBC_PASSWORD, "password"); DataSource ds =
 * dsf.createDataSource(props);
 * 
 * This service also supports a URL-based data source. The following 3
 * properties can be provided instead of the 5 properties above:
 * 
 * props.put(DataSourceFactory.JDBC_URL, "jdbc:mysql://localhost:1527/myDB");
 * props.put(DataSourceFactory.JDBC_USER, "mike");
 * props.put(DataSourceFactory.JDBC_PASSWORD, "password");
 */
public final class MysqlDataSourceFactory extends AbstractDataSourceFactory {

    public MysqlDataSourceFactory() {
        /**
         * Nothing to do
         */
    }

    @Override
    public DataSource createDataSource(final Properties props) throws SQLException {
        if (props.get(MysqlDataSourceFactory.JDBC_URL) != null) {
            try {
                return new UrlBasedDriverDataSource(props);
            } catch (Exception e) {
                throw new SQLException("Unable to create datasource", e);
            }
        } else {
            DataSource dataSource = new MysqlConnectionPoolDataSource();
            setDataSourceProperties(dataSource, safeGetProperties(props));
            return dataSource;
        }
    }

    @Override
    public ConnectionPoolDataSource createConnectionPoolDataSource(final Properties props) throws SQLException {
        ConnectionPoolDataSource dataSource = new MysqlConnectionPoolDataSource();
        setDataSourceProperties(dataSource, safeGetProperties(props));
        return dataSource;
    }

    @Override
    public XADataSource createXADataSource(final Properties props) throws SQLException {
        XADataSource dataSource = new MysqlXADataSource();
        setDataSourceProperties(dataSource, safeGetProperties(props));
        return dataSource;
    }

    @Override
    public Driver createDriver(final Properties props) throws SQLException {
        System.out.println("CREATE DRIVER");
        // Properties not used when accessing the raw driver.
        Driver driver = new com.mysql.jdbc.Driver();
        setDataSourceProperties(driver, safeGetProperties(props));
        return driver;
    }

    private Properties safeGetProperties(final Properties props) {
        if (props == null) {
            return new Properties();
        } else {
            return props;
        }
    }
}